/// Provides all functionality to list all open imports.
use crate::archive::Archive;

use anyhow::Result;

use crate::error::Archive as ArchiveError;

/// List all open imports
pub fn list(archive: Archive) -> Result<(), ArchiveError> {
    // let import_branches: Result<Vec<String>, ArchiveError> = archive
    let import_branches: Result<Vec<String>, ArchiveError> = archive
        .get_imports()?
        .into_iter()
        .filter_map(|branch| -> Option<Result<String, ArchiveError>> {
            // Returns:
            //    None to filter out element
            //    Some(Ok(elemem)) to include element
            //    Some(Err) to make the overall variable Err
            let branch_name = branch.name();
            // TODO: cleanup match
            match branch_name {
                Ok(b) => {
                    let b2 = b.expect("Branch name contains invalid utf-8 chars");
                    let a = b2.strip_prefix(&archive.config.general.import_branch_prefix);
                    match a {
                        Some(x) => Some(Ok(x.to_string())),
                        None => None,
                    }
                }
                Err(e) => Some(Err(ArchiveError::from(e))),
            }
        })
        .collect();
    let import_branches = import_branches?;

    if import_branches.is_empty() {
        println!("There are no open imports!");
        Ok(())
    } else {
        println!("Open imports:");
        for b in import_branches {
            println!("{}", b);
        }
        Ok(())
    }
}
