/// Struct representing, and functions for working, with format strings.
use chrono::prelude::NaiveDateTime;

use super::{fmt_count, fmt_extension, fmt_identifier, fmt_start_end_datetime, FSState};
use crate::config;

#[derive(Debug)]
pub struct BranchFS {
    raw: String,
    pub state: FSState,
}

impl BranchFS {
    /// Initialize `FormatString` representing import branch.
    ///
    /// The schema is read from the provided config.
    pub fn init(
        config: &config::Config,
        start_dt: Option<NaiveDateTime>,
        end_dt: Option<NaiveDateTime>,
        identifier: Option<String>,
    ) -> Self {
        Self {
            raw: format!(
                "{}{}",
                config.general.import_branch_prefix, config.general.import_branch_schema
            ),
            state: FSState::init(None, start_dt, end_dt, identifier, None, None),
        }
    }

    /// Get fully resolved format string.
    pub fn format(&self) -> String {
        let out = self.raw.clone();
        let out = fmt_start_end_datetime(&out, &self.state.start_dt, &self.state.end_dt);
        let out = fmt_identifier(&out, &self.state.identifier);
        let out = fmt_extension(&out, &self.state.extension);
        let out = fmt_count(&out, &self.state.count.get());
        println!("Formatted {} to {}", self.raw, out);
        out
    }
}
