#![feature(absolute_path)]
#![feature(let_chains)] // git::switch_branch
#![feature(assert_matches)] // replaces `assert!(matches!(x, y))` in tests
mod archive;
mod commands;
mod config;
mod error;
mod format_string;
mod git;
mod metadata;

#[cfg(test)]
mod conftest;

use anyhow::Result;
use clap::Parser;

use commands::{Cli, Commands};

fn main() -> Result<()> {
    let cli = Cli::parse();

    Ok(match cli.command {
        Commands::Config => {
            println!("Command Config");
            commands::config::run()?;
        }
        Commands::Cull => {
            println!("Command Cull");
            commands::cull::run()?;
        }
        Commands::Init => {
            println!("Command Initialize");
            commands::init::run()?;
        }
        Commands::Import(arg) => {
            println!("Command Import");
            commands::import::run(arg)?;
        }
    })
}
