/// Provided metadata related functionality
use std::fs::File;
use std::io::BufReader;
use std::path::PathBuf;

use exif::{Exif, Field, Reader, Tag, Value};

use super::error::Metadata as MetadataError;

/// Holds `Exif` which is use interact with an images metadata.
pub struct Metadata {
    exif: Exif,
}

impl Metadata {
    /// Initialize `Metadata` by creating an instance of `Exif`.
    pub fn init(path: &PathBuf) -> Result<Self, MetadataError> {
        let file = File::open(path)?;
        let mut bufreader = BufReader::new(&file);
        let exifreader = Reader::new();
        let exif = exifreader.read_from_container(&mut bufreader)?;

        Ok(Self { exif })
    }

    fn get_field(&self, tag: Tag) -> Result<&Field, MetadataError> {
        self.exif
            .get_field(tag, exif::In::PRIMARY)
            .ok_or(MetadataError::InvalidField {
                msg: format!("{}", tag),
            })
    }

    /// Returns the raw metadata value associated to `tag`.
    pub fn get_raw(&self, tag: Tag) -> Result<&Value, MetadataError> {
        Ok(&self.get_field(tag)?.value)
    }

    /// Returns a nicely formatted metadata value associated to `tag`.
    pub fn get_formatted(&self, tag: Tag) -> Result<String, MetadataError> {
        Ok(self
            .get_field(tag)?
            .display_value()
            .with_unit(&self.exif)
            .to_string())
    }
}
